@@ -19,12 +19,52 @@ from astropy.visualization import PercentileInterval
 from astroquery.skyview import SkyView
 from astropy.utils.data import clear_download_cache
 
-from utils import write_products, write_bytesio_to_file, write_zipped_fits_file
-
 Run, Instance, Detection, Product, Source  = None, None, None, None, None
 SourceDetection, Comment, Tag, TagDetection, TagSourceDetection = None, None, None, None, None
 
 
+# utils
+def _write_bytesio_to_file(filename, bytesio):
+    """Write the contents of the given BytesIO to a file.
+    Creates the file or overwrites the file if it does
+    not exist yet. 
+    
+    """
+    with open(filename, "wb") as outfile:
+        # Copy the BytesIO stream to the output file
+        outfile.write(bytesio.getbuffer())
+            
+
+def _write_zipped_fits_file(filename, product, compress=True):
+    """Compress a .fits file as .fits.gz for a data product.
+   
+    """
+    with io.BytesIO() as buf:
+        buf.write(product)
+        buf.seek(0)
+        if not os.path.isfile(filename):
+            _write_bytesio_to_file(filename, buf)
+            if compress:
+                os.system(f'gzip {filename}')
+
+
+def _write_products(products, prefix):
+    _write_zipped_fits_file('%s_cube.fits' % (prefix), products.cube)
+    _write_zipped_fits_file('%s_chan.fits' % (prefix), products.chan)
+    _write_zipped_fits_file('%s_mask.fits' % (prefix), products.mask)
+    _write_zipped_fits_file('%s_mom0.fits' % (prefix), products.mom0)
+    _write_zipped_fits_file('%s_mom1.fits' % (prefix), products.mom1)
+    _write_zipped_fits_file('%s_mom2.fits' % (prefix), products.mom2)
+
+    # Open spectrum
+    with io.BytesIO() as buf:
+        buf.write(b''.join(products.spec))
+        buf.seek(0)
+        spec_file  = '%s_spec.txt' % (prefix)
+        if not os.path.isfile(spec_file):
+            _write_bytesio_to_file(spec_file, buf)
+
+
 # Connect to WALLABY database
 def connect():
     global Run, Instance, Detection, Product, Source
@@ -142,7 +182,7 @@ def save_products_for_source(tag, source_name, *args, **kwargs):
         os.mkdir(parent)
     
     # Write fits files
-    write_products(products, f'{parent}/{name}')
+    _write_products(products, f'{parent}/{name}')
     
     return
 
@@ -162,7 +202,7 @@ def save_products(tag, *args, **kwargs):
             os.mkdir(f'{parent}/{name}')
         detection = Detection.objects.get(id=row['id'])
         products = Product.objects.get(detection=detection)
-        write_products(products, f'{parent}/{name}/{name}')
+        _write_products(products, f'{parent}/{name}/{name}')
 
     os.system(f'tar -czf {parent}.tar.gz {parent}')
 
@@ -320,3 +360,53 @@ def save_overview(tag, *args, **kwargs):
     os.system(f'tar -czf {parent}.tar.gz {parent}')
 
     return
+
+
+def parse_spectrum_to_table(spectrum):
+    """Takes the spectrum stored in the database and parses the object to an Astropy.table
+    Columns: Channel, Frequency, Flux density, Pixels
+    
+    """
+    array = []
+    with io.BytesIO() as buf:
+        buf.write(b''.join(spectrum))
+        buf.seek(0)
+        text = buf.getbuffer().tobytes().decode('utf-8')
+        lines = text.strip().split('\n')
+        for line in lines:
+            if not line.startswith('#'):
+                chan, freq, flux, pix = line.strip().split()
+                array.append(np.array([int(chan), float(freq), float(flux), int(pix)]))
+
+    t = Table(
+        np.array(array),
+        names=('Channel', 'Frequency', 'Flux Density', 'Pixels'),
+        dtype=(int, np.float32, np.float32, int)
+    )
+    return t
+
+
+def casda_export_products(table, directory):
+    """Export data products for sources in an astropy.Table object
+    to an output directory in a format compatible for CASDA ingest.
+
+    """
+    for row in table:
+        name = row['name'].replace(' ', '_')
+        detection = Detection.objects.get(id=row['id'])
+        products = Product.objects.get(detection=detection)
+        filename_prefix = f'{directory}/{name}'
+        
+        # write .fits files
+        _write_zipped_fits_file('%s_cube.fits' % (filename_prefix), products.cube, compress=False)
+        _write_zipped_fits_file('%s_chan.fits' % (filename_prefix), products.chan, compress=False)
+        _write_zipped_fits_file('%s_mask.fits' % (filename_prefix), products.mask, compress=False)
+        _write_zipped_fits_file('%s_mom0.fits' % (filename_prefix), products.mom0, compress=False)
+        _write_zipped_fits_file('%s_mom1.fits' % (filename_prefix), products.mom1, compress=False)
+        _write_zipped_fits_file('%s_mom2.fits' % (filename_prefix), products.mom2, compress=False)
+
+        # write spectrum as fits file
+        spectrum_table = parse_spectrum_to_table(products.spec)
+        spectrum_tables.write('%s_spec.fits' % (filename_prefix), format='fits')
+
+    return
\ No newline at end of file
